def RandRows(d,a,n): return Mat(ZZ,n,n-1).random_element(density=d, x = a, y = -a + 1)

def RandElem(d,a,M): return M * Mat(ZZ,M.ncols(),1).random_element(x = -a, y = 1)

def RandPoint(d,a,n): return Mat(ZZ,n,1).random_element(x = -a, y = a)

def AddColumn(M,b): return matrix([list(M[i])+list(b[i]) for i in range(0,M.nrows())])
def AddRow(M,a): return matrix([list(M[i]) for i in range(0,M.nrows())] + list(a))
def Copy(M): return matrix([list(M[i]) for i in range(0,M.nrows())])

def MatrixToILP(A,b):
    n = len(A.columns())
    p = MixedIntegerLinearProgram()
    v = p.new_variable(integer=True,nonnegative=False)
    p.set_objective(sum([A[0][j]*v[j] for j in range(0,n)]))
    for i in range(0,A.nrows()):
        p.add_constraint(b[i][0] <= sum([A[i][j]*v[j] for j in range(0,n)]) )
    return p
    
def MatrixToLP(A,b):
    n = len(A.columns())
    p = MixedIntegerLinearProgram()
    v = p.new_variable(real=True,nonnegative=False)
    p.set_objective(sum([A[0][j]*v[j] for j in range(0,n)]))
    for i in range(0,A.nrows()):
        p.add_constraint(b[i][0] <= sum([A[i][j]*v[j] for j in range(0,n)]) )
    return p
    
def MatrixToLPExternal(A,b,p,v):
    n = len(A.columns())
    p.set_objective(sum([A[0][j]*v[j] for j in range(0,n)]))
    for i in range(0,A.nrows()):
        p.add_constraint(b[i][0] <= sum([A[i][j]*v[j] for j in range(0,n)]) )
    return p
    
def MatrixToUnitCube(A,b):
    n = len(A.columns())
    p = MixedIntegerLinearProgram()
    v = p.new_variable(real=True,nonnegative=False)
    p.set_objective(sum([A[0][j]*v[j] for j in range(0,n)]))
    for i in range(0,A.nrows()):
        p.add_constraint(b[i][0] + sum([abs(A[0][j]) for j in range(0,n)])*0.5 <= sum([A[i][j]*v[j] for j in range(0,n)]) )
    return p
    
def MatrixToEQ(A,b):
    n = len(A.columns())
    p = MixedIntegerLinearProgram()
    v = p.new_variable(real=True,nonnegative=False)
    p.set_objective(sum([A[0][j]*v[j] for j in range(0,n)]))
    for i in range(0,A.nrows()):
        p.add_constraint(b[i][0] == sum([A[i][j]*v[j] for j in range(0,n)]) )
    return p

def OutOfRange(a,n,v):
    for i in range(0,n):
        if v[i][0] >= n*a : return true
        if v[i][0] <= -n*a : return true
    return false
    
def BuildRows(d,a,n,m):
    M = RandRows(d,a,n)
    while M.ncols() < m:
        v = RandElem(d,a,M)
        if not(OutOfRange(a,n,v)):
            M = AddColumn(M,v)
    return M.transpose()
    
def BuildBounds(d,a,n,m,A):
    while true:
        v = RandPoint(d,a,n)
        b = A * v + RandPoint(d,1,m)
        try:
            MatrixToLP(A,b).solve()
        except sage.numerical.mip.MIPSolverException:
            continue
        try:
            MatrixToEQ(A,b).solve()
            continue
        except sage.numerical.mip.MIPSolverException:
            try:
                MatrixToUnitCube(A,b).solve()
            except sage.numerical.mip.MIPSolverException:
                return b
    return RandPoint(d,n*a,m)    

    
def MatrixToDFG(A,b,i,folder):
    f = file(folder+'/unbd-sage'+str(i)+'.dfg','w')
    s = "variables[ "
    for j in range(0,A.ncols()):
        s = s + "x"+str(j)
        if j < A.ncols() - 1:
            s = s + ", "
    f.write(s + "].")
    f.write("\nand( ")
    for i in range(0,A.nrows()):
        s = ""
        s = s + "\nle(" + str(b[i][0]) +", "
        nnonz = 0
        for j in range(0,A.ncols()):
            if A[i][j] != 0:
                nnonz = nnonz + 1
        k = 0
        for j in range(0,A.ncols()):
            if A[i][j] != 0:
                if k < nnonz - 1:
                    s = s + "plus(mult(" + str(A[i][j]) + ", x" + str(j) + "), "
                    k = k + 1
                else:
                    s = s + "mult(" + str(A[i][j]) + ", x" + str(j) + ")"
        if nnonz == 0:
            s = s + "0"
        for k in range(0,nnonz):
            s = s + ")"
        if(i < A.nrows() - 1):
            f.write(s + ",")
        else:
            f.write(s + ")")
    f.close()
    
def MatrixToSMT(A,b,i,folder):
    f = file(folder+'/unbd-sage'+str(i)+'.smt','w')
    s = "(set-logic QF_LIA)\n"
    for j in range(0,A.ncols()):
        s = s + "(declare-fun x"+str(j)+" () Int)\n"
    f.write(s)
    f.write("\n(assert (and ")
    for i in range(0,A.nrows()):
        s = ""
        if b[i][0] >= 0:
            s = s + "\n(<= " + str(b[i][0]) +" "
        else:
            s = s + "\n(<= (- " + str(-b[i][0]) +") "
        nnonz = 0
        for j in range(0,A.ncols()):
            if A[i][j] != 0:
                nnonz = nnonz + 1
        k = 0
        for j in range(0,A.ncols()):
            if A[i][j] > 0:
                if k < nnonz - 1:
                    s = s + "(+ (* " + str(A[i][j]) + " x" + str(j) + ") "
                    k = k + 1
                else:
                    s = s + "(* " + str(A[i][j]) + " x" + str(j) + ") "
            if A[i][j] < 0:
                if k < nnonz - 1:
                    s = s + "(+ (* (- " + str(-A[i][j]) + ") x" + str(j) + ") "
                    k = k + 1
                else:
                    s = s + "(* (- " + str(-A[i][j]) + ") x" + str(j) + ") "
        if nnonz == 0:
            s = s + "0)"
        for k in range(0,nnonz):
            s = s + ")"
        if(i < A.nrows() - 1):
            f.write(s + " ")
        else:
            f.write(s + "))")
    f.write("\n(check-sat)")
    f.write("\n(exit)")
    f.close()

def generate(folder,number,variables,constraints,maximum,density):
    r"""
       Randomly generates <number> partially unbounded and satisfiable linear integer problems in the smt-lib format.

       INPUT:

       - ``folder`` -- string; folder in which the problems are generated 

       - ``number`` -- integer; number of variables in each problem

       - ``variables`` -- integer; number of variables in each problem

       - ``constraints`` -- integer; number of constraints in each problem

       - ``maximum`` -- integer; maximum absolute value of any randomly generated number

       - ``density`` -- float; density of each randomly generated vector (0.0: only zero entries, 1.0: no zero entries)
    """
    for i in range(0,number):
        A = BuildRows(density,maximum,variables,constraints)
        b = BuildBounds(density,maximum,variables,constraints,A)
        MatrixToSMT(A,b,i,folder)